from decimal import Decimal

import flask

import flask_babel as babel


def test_basics():
    app = flask.Flask(__name__)
    babel.Babel(app)
    n = 1099

    with app.test_request_context():
        assert babel.format_number(n) == u'1,099'
        assert babel.format_decimal(Decimal('1010.99')) == u'1,010.99'
        assert babel.format_currency(n, 'USD') == '$1,099.00'
        assert babel.format_percent(0.19) == '19%'
        assert babel.format_scientific(10000) == u'1E4'



import pickle

import flask
from babel.support import NullTranslations

import flask_babel as babel
from flask_babel import get_translations, gettext, lazy_gettext


def test_no_request_context():
    b = babel.Babel()
    app = flask.Flask(__name__)
    b.init_app(app)

    with app.app_context():
        assert isinstance(get_translations(), NullTranslations)


def test_multiple_directories():
    """
    Ensure we can load translations from multiple directories.

    This also ensures that directories without any translation files
    are not taken into account.
    """
    b = babel.Babel()
    app = flask.Flask(__name__)

    app.config.update({
        'BABEL_TRANSLATION_DIRECTORIES': ';'.join((
            'translations',
            'renamed_translations'
        )),
        'BABEL_DEFAULT_LOCALE': 'de_DE'
    })

    b.init_app(app)

    with app.test_request_context():
        translations = b.list_translations()

        assert(len(translations) == 3)
        assert(str(translations[0]) == 'de')
        assert(str(translations[1]) == 'de')
        assert(str(translations[2]) == 'de_DE')

        assert gettext(
            u'Hello %(name)s!',
            name='Peter'
        ) == 'Hallo Peter!'


def test_multiple_directories_multiple_domains():
    """
    Ensure we can load translations from multiple directories with a
    custom domain.
    """
    b = babel.Babel()
    app = flask.Flask(__name__)

    app.config.update({
        'BABEL_TRANSLATION_DIRECTORIES': ';'.join((
            'renamed_translations',
            'translations_different_domain',
        )),
        'BABEL_DEFAULT_LOCALE': 'de_DE',
        'BABEL_DOMAIN': ';'.join((
            'messages',
            'myapp',
        )),
    })

    b.init_app(app)

    with app.test_request_context():
        translations = b.list_translations()

        assert(len(translations) == 3)
        assert(str(translations[0]) == 'de')
        assert(str(translations[1]) == 'de')
        assert(str(translations[2]) == 'de_DE')

        assert gettext(
            u'Hello %(name)s!',
            name='Peter'
        ) == 'Hallo Peter!'
        assert gettext(u'Good bye') == 'Auf Wiedersehen'


def test_multiple_directories_different_domain():
    """
    Ensure we can load translations from multiple directories with a
    custom domain.
    """
    b = babel.Babel()
    app = flask.Flask(__name__)

    app.config.update({
        'BABEL_TRANSLATION_DIRECTORIES': ';'.join((
            'translations_different_domain',
            'renamed_translations'
        )),
        'BABEL_DEFAULT_LOCALE': 'de_DE',
        'BABEL_DOMAIN': 'myapp'
    })

    b.init_app(app)

    with app.test_request_context():
        translations = b.list_translations()

        assert(len(translations) == 3)
        assert(str(translations[0]) == 'de')
        assert(str(translations[1]) == 'de')
        assert(str(translations[2]) == 'de_DE')

        assert gettext(
            u'Hello %(name)s!',
            name='Peter'
        ) == 'Hallo Peter!'
        assert gettext(u'Good bye') == 'Auf Wiedersehen'


def test_different_domain():
    """
    Ensure we can load translations from multiple directories.
    """
    b = babel.Babel()
    app = flask.Flask(__name__)

    app.config.update({
        'BABEL_TRANSLATION_DIRECTORIES': 'translations_different_domain',
        'BABEL_DEFAULT_LOCALE': 'de_DE',
        'BABEL_DOMAIN': 'myapp'
    })

    b.init_app(app)

    with app.test_request_context():
        translations = b.list_translations()

        assert(len(translations) == 2)
        assert(str(translations[0]) == 'de')
        assert(str(translations[1]) == 'de_DE')

        assert gettext(u'Good bye') == 'Auf Wiedersehen'


def test_lazy_old_style_formatting():
    lazy_string = lazy_gettext(u'Hello %(name)s')
    assert lazy_string % {u'name': u'test'} == u'Hello test'

    lazy_string = lazy_gettext(u'test')
    assert u'Hello %s' % lazy_string == u'Hello test'


def test_lazy_pickling():
    lazy_string = lazy_gettext(u'Foo')
    pickled = pickle.dumps(lazy_string)
    unpickled = pickle.loads(pickled)

    assert unpickled == lazy_string



from datetime import datetime, timedelta

import flask

import flask_babel as babel
from flask_babel import get_babel


def test_basics():
    app = flask.Flask(__name__)
    babel.Babel(app)
    d = datetime(2010, 4, 12, 13, 46)
    delta = timedelta(days=6)

    with app.test_request_context():
        assert babel.format_datetime(d) == 'Apr 12, 2010, 1:46:00\u202fPM'
        assert babel.format_date(d) == 'Apr 12, 2010'
        assert babel.format_time(d) == '1:46:00\u202fPM'
        assert babel.format_timedelta(delta) == '1 week'
        assert babel.format_timedelta(delta, threshold=1) == '6 days'

    with app.test_request_context():
        get_babel(app).default_timezone = 'Europe/Vienna'
        assert babel.format_datetime(d) == 'Apr 12, 2010, 3:46:00\u202fPM'
        assert babel.format_date(d) == 'Apr 12, 2010'
        assert babel.format_time(d) == '3:46:00\u202fPM'

    with app.test_request_context():
        get_babel(app).default_locale = 'de_DE'
        assert babel.format_datetime(d, 'long') == \
            '12. April 2010, 15:46:00 MESZ'


def test_custom_formats():
    app = flask.Flask(__name__)
    app.config.update(
        BABEL_DEFAULT_LOCALE='en_US',
        BABEL_DEFAULT_TIMEZONE='Pacific/Johnston'
    )
    b = babel.Babel(app)
    b.date_formats['datetime'] = 'long'
    b.date_formats['datetime.long'] = 'MMMM d, yyyy h:mm:ss a'
    d = datetime(2010, 4, 12, 13, 46)

    with app.test_request_context():
        assert babel.format_datetime(d) == 'April 12, 2010 3:46:00 AM'


def test_custom_locale_selector():
    app = flask.Flask(__name__)
    b = babel.Babel(app)
    d = datetime(2010, 4, 12, 13, 46)

    the_timezone = 'UTC'
    the_locale = 'en_US'

    def select_locale():
        return the_locale

    def select_timezone():
        return the_timezone

    get_babel(app).locale_selector = select_locale
    get_babel(app).timezone_selector = select_timezone

    with app.test_request_context():
        assert babel.format_datetime(d) == 'Apr 12, 2010, 1:46:00\u202fPM'

    the_locale = 'de_DE'
    the_timezone = 'Europe/Vienna'

    with app.test_request_context():
        assert babel.format_datetime(d) == '12.04.2010, 15:46:00'


def test_refreshing():
    app = flask.Flask(__name__)
    babel.Babel(app)
    d = datetime(2010, 4, 12, 13, 46)
    with app.test_request_context():
        assert babel.format_datetime(d) == 'Apr 12, 2010, 1:46:00\u202fPM'
        get_babel(app).default_timezone = 'Europe/Vienna'
        babel.refresh()
        assert babel.format_datetime(d) == 'Apr 12, 2010, 3:46:00\u202fPM'



from threading import Semaphore, Thread

import flask

import flask_babel as babel


def test_force_locale():
    app = flask.Flask(__name__)
    babel.Babel(app, locale_selector=lambda: 'de_DE')

    with app.test_request_context():
        assert str(babel.get_locale()) == 'de_DE'
        with babel.force_locale('en_US'):
            assert str(babel.get_locale()) == 'en_US'
        assert str(babel.get_locale()) == 'de_DE'


def test_force_locale_with_threading():
    app = flask.Flask(__name__)
    babel.Babel(app, locale_selector=lambda: 'de_DE')

    semaphore = Semaphore(value=0)

    def first_request():
        with app.test_request_context():
            with babel.force_locale('en_US'):
                assert str(babel.get_locale()) == 'en_US'
                semaphore.acquire()

    thread = Thread(target=first_request)
    thread.start()

    try:
        with app.test_request_context():
            assert str(babel.get_locale()) == 'de_DE'
    finally:
        semaphore.release()
        thread.join()


def test_force_locale_with_threading_and_app_context():
    app = flask.Flask(__name__)
    babel.Babel(app, locale_selector=lambda: 'de_DE')

    semaphore = Semaphore(value=0)

    def first_app_context():
        with app.app_context():
            with babel.force_locale('en_US'):
                assert str(babel.get_locale()) == 'en_US'
                semaphore.acquire()

    thread = Thread(target=first_app_context)
    thread.start()

    try:
        with app.app_context():
            assert str(babel.get_locale()) == 'de_DE'
    finally:
        semaphore.release()
        thread.join()


def test_refresh_during_force_locale():
    app = flask.Flask(__name__)
    babel.Babel(app, locale_selector=lambda: 'de_DE')

    with app.test_request_context():
        with babel.force_locale('en_US'):
            assert str(babel.get_locale()) == 'en_US'
            babel.refresh()
            assert str(babel.get_locale()) == 'en_US'



import flask
import flask_babel as babel


def test_multiple_apps():
    b = babel.Babel()

    app1 = flask.Flask(__name__)
    b.init_app(app1, default_locale='de_DE')

    app2 = flask.Flask(__name__)
    b.init_app(app2, default_locale='en_US')

    with app1.test_request_context():
        assert str(babel.get_locale()) == 'de_DE'
        assert babel.gettext(u'Hello %(name)s!', name='Peter') == \
               'Hallo Peter!'

    with app2.test_request_context():
        assert str(babel.get_locale()) == 'en_US'
        assert babel.gettext(u'Hello %(name)s!', name='Peter') == \
               'Hello Peter!'



import flask
import flask_babel as babel


def test_app_factory():
    b = babel.Babel()

    def locale_selector():
        return 'de_DE'

    def create_app():
        app_ = flask.Flask(__name__)
        b.init_app(
            app_,
            default_locale='en_US',
            locale_selector=locale_selector
        )
        return app_

    app = create_app()
    with app.test_request_context():
        assert str(babel.get_locale()) == 'de_DE'
        assert babel.gettext(u'Hello %(name)s!', name='Peter') == \
               'Hallo Peter!'



import flask

import flask_babel as babel
from flask_babel import (
    gettext,
    lazy_gettext,
    lazy_ngettext,
    ngettext,
    get_babel
)


def test_basics():
    app = flask.Flask(__name__)
    babel.Babel(app, default_locale='de_DE')

    with app.test_request_context():
        assert gettext(u'Hello %(name)s!', name='Peter') == 'Hallo Peter!'
        assert ngettext(u'%(num)s Apple', u'%(num)s Apples', 3) == \
            u'3 Äpfel'
        assert ngettext(u'%(num)s Apple', u'%(num)s Apples', 1) == \
            u'1 Apfel'


def test_template_basics():
    app = flask.Flask(__name__)
    babel.Babel(app, default_locale='de_DE')

    def t(x):
        return flask.render_template_string('{{ %s }}' % x)

    with app.test_request_context():
        assert t("gettext('Hello %(name)s!', name='Peter')") == \
            u'Hallo Peter!'
        assert t("ngettext('%(num)s Apple', '%(num)s Apples', 3)") == \
            u'3 Äpfel'
        assert t("ngettext('%(num)s Apple', '%(num)s Apples', 1)") == \
            u'1 Apfel'
        assert flask.render_template_string('''
            {% trans %}Hello {{ name }}!{% endtrans %}
        ''', name='Peter').strip() == 'Hallo Peter!'
        assert flask.render_template_string('''
            {% trans num=3 %}{{ num }} Apple
            {%- pluralize %}{{ num }} Apples{% endtrans %}
        ''', name='Peter').strip() == u'3 Äpfel'


def test_lazy_gettext():
    app = flask.Flask(__name__)
    babel.Babel(app, default_locale='de_DE')
    yes = lazy_gettext(u'Yes')
    with app.test_request_context():
        assert str(yes) == 'Ja'
        assert yes.__html__() == 'Ja'

    get_babel(app).default_locale = 'en_US'
    with app.test_request_context():
        assert str(yes) == 'Yes'
        assert yes.__html__() == 'Yes'


def test_lazy_ngettext():
    app = flask.Flask(__name__)
    babel.Babel(app, default_locale='de_DE')
    one_apple = lazy_ngettext(u'%(num)s Apple', u'%(num)s Apples', 1)
    with app.test_request_context():
        assert str(one_apple) == '1 Apfel'
        assert one_apple.__html__() == '1 Apfel'
    two_apples = lazy_ngettext(u'%(num)s Apple', u'%(num)s Apples', 2)
    with app.test_request_context():
        assert str(two_apples) == u'2 Äpfel'
        assert two_apples.__html__() == u'2 Äpfel'


def test_lazy_gettext_defaultdomain():
    app = flask.Flask(__name__)
    babel.Babel(app, default_locale='de_DE', default_domain='test')
    first = lazy_gettext('first')

    with app.test_request_context():
        assert str(first) == 'erste'

    get_babel(app).default_locale = 'en_US'
    with app.test_request_context():
        assert str(first) == 'first'


def test_list_translations():
    app = flask.Flask(__name__)
    b = babel.Babel(app, default_locale='de_DE')

    with app.app_context():
        translations = b.list_translations()
        assert len(translations) == 2
        assert str(translations[0]) == 'de'
        assert str(translations[1]) == 'de_DE'


def test_list_translations_default_locale_exists():
    app = flask.Flask(__name__)
    b = babel.Babel(app, default_locale='de')

    with app.app_context():
        translations = b.list_translations()
        assert len(translations) == 1
        assert str(translations[0]) == 'de'


def test_no_formatting():
    """
    Ensure we don't format strings unless a variable is passed.
    """
    app = flask.Flask(__name__)
    babel.Babel(app)

    with app.test_request_context():
        assert gettext(u'Test %s') == u'Test %s'
        assert gettext(u'Test %(name)s', name=u'test') == u'Test test'
        assert gettext(u'Test %s') % 'test' == u'Test test'


def test_domain():
    app = flask.Flask(__name__)
    b = babel.Babel(app, default_locale='de_DE')
    domain = babel.Domain(domain='test')

    with app.test_request_context():
        assert domain.gettext('first') == 'erste'
        assert babel.gettext('first') == 'first'


def test_as_default():
    app = flask.Flask(__name__)
    b = babel.Babel(app, default_locale='de_DE')
    domain = babel.Domain(domain='test')

    with app.test_request_context():
        assert babel.gettext('first') == 'first'
        domain.as_default()
        assert babel.gettext('first') == 'erste'


def test_default_domain():
    app = flask.Flask(__name__)
    b = babel.Babel(app, default_locale='de_DE', default_domain='test')

    with app.test_request_context():
        assert babel.gettext('first') == 'erste'


def test_multiple_apps():
    app1 = flask.Flask(__name__)
    b1 = babel.Babel(app1, default_locale='de_DE')

    app2 = flask.Flask(__name__)
    b2 = babel.Babel(app2, default_locale='de_DE')

    with app1.test_request_context() as ctx:
        assert babel.gettext('Yes') == 'Ja'

        assert ('de_DE', 'messages') in b1.domain_instance.get_translations_cache(ctx)

    with app2.test_request_context() as ctx:
        assert 'de_DE', 'messages' not in b2.domain_instance.get_translations_cache(ctx)


def test_cache(mocker):
    load_mock = mocker.patch(
        "babel.support.Translations.load",
        side_effect=babel.support.Translations.load
    )

    app = flask.Flask(__name__)
    b = babel.Babel(
        app,
        default_locale="de_DE",
        locale_selector=lambda: the_locale
    )

    # first request, should load en_US
    the_locale = "en_US"
    with app.test_request_context() as ctx:
        assert b.domain_instance.get_translations_cache(ctx) == {}
        assert babel.gettext("Yes") == "Yes"
    assert load_mock.call_count == 1

    # second request, should use en_US from cache
    with app.test_request_context() as ctx:
        assert set(b.domain_instance.get_translations_cache(ctx)) == {
            ("en_US", "messages")
        }
        assert babel.gettext("Yes") == "Yes"
    assert load_mock.call_count == 1

    # third request, should load de_DE from cache
    the_locale = "de_DE"
    with app.test_request_context() as ctx:
        assert set(b.domain_instance.get_translations_cache(ctx)) == {
            ("en_US", "messages")
        }
        assert babel.gettext("Yes") == "Ja"
    assert load_mock.call_count == 2

    # now everything is cached, so no more loads should happen!
    the_locale = "en_US"
    with app.test_request_context() as ctx:
        assert set(b.domain_instance.get_translations_cache(ctx)) == {
            ("en_US", "messages"),
            ("de_DE", "messages"),
        }
        assert babel.gettext("Yes") == "Yes"
    assert load_mock.call_count == 2

    the_locale = "de_DE"
    with app.test_request_context() as ctx:
        assert set(b.domain_instance.get_translations_cache(ctx)) == {
            ("en_US", "messages"),
            ("de_DE", "messages"),
        }
        assert babel.gettext("Yes") == "Ja"
    assert load_mock.call_count == 2
